﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Grouping Vector Manipulator", "An operator which manipulates a PackingSequence representation.")]
  [StorableClass]
  public abstract class GroupingVectorManipulator : PackingSolutionManipulator, IGroupingVectorOperator {
    [StorableConstructor]
    protected GroupingVectorManipulator(bool deserializing) : base(deserializing) { }
    protected GroupingVectorManipulator(GroupingVectorManipulator original, Cloner cloner) : base(original, cloner) { }
    public GroupingVectorManipulator()
      : base() {
        EncodedSolutionParameter.ActualName = "GroupingVector";
    }

    protected abstract void Manipulate(IRandom random, GroupingVectorEncoding individual);
    public override IOperation Apply() {
      var solution = EncodedSolutionParameter.ActualValue as GroupingVectorEncoding;
      if (solution == null) throw new InvalidOperationException("PackingSolutionEncoding was not found or is not of type GroupingVectorEncoding.");
      Manipulate(RandomParameter.ActualValue, solution);
      return base.Apply();
    }

  }
}
